@@ -27,6 +27,7 @@
 #include <libgda/gda-config.h>
 #include <libgda/gda-intl.h>
 #include <libgda/gda-log.h>
+#include <string.h>
 #include "gda-marshal.h"
 
 #define PARENT_TYPE G_TYPE_OBJECT
@@ -59,6 +60,32 @@ enum {
 static gint gda_client_signals[LAST_SIGNAL] = { 0 };
 static GObjectClass *parent_class = NULL;
 
+/*
+ * Private functions
+ */
+
+static void
+emit_client_error (GdaClient *client, GdaConnection *cnc, const gchar *format, ...)
+{
+	va_list args;
+	gchar sz[2048];
+	GdaError *error;
+	GList *list;
+
+	/* build the message string */
+	va_start (args, format);
+	vsprintf (sz, format, args);
+	va_end (args);
+
+	/* create the error list */
+	error = gda_error_new ();
+	gda_error_set_description (error, sz);
+	gda_error_set_source (error, "[GDA client]");
+
+	list = g_list_append (NULL, error);
+	g_signal_emit (G_OBJECT (client), gda_client_signals[ERROR], 0, cnc, list);
+}
+
 /*
  * Callbacks
  */
@@ -241,7 +268,9 @@ gda_client_open_connection (GdaClient *client,
 
 		prv_info = gda_config_get_provider_by_name (dsn_info->provider);
 		if (!prv_info) {
-			gda_log_error (_("Provider %s is not installed"), dsn_info->provider);
+			emit_client_error (client, NULL,
+					   _("Could not find provider %s in the current setup"),
+					   dsn_info->provider);
 			gda_config_free_data_source_info (dsn_info);
 			return NULL;
 		}
@@ -250,7 +279,7 @@ gda_client_open_connection (GdaClient *client,
 		prv = g_new0 (LoadedProvider, 1);
 		prv->handle = g_module_open (prv_info->location, G_MODULE_BIND_LAZY);
 		if (!prv->handle) {
-			gda_log_error (_("Could not load %s provider"), dsn_info->provider);
+			emit_client_error (client, NULL, g_module_error ());
 			gda_config_free_data_source_info (dsn_info);
 			g_free (prv);
 			return NULL;
@@ -266,8 +295,9 @@ gda_client_open_connection (GdaClient *client,
 				 (gpointer) &prv->plugin_create_provider);
 
 		if (!prv->plugin_create_provider) {
-			gda_log_error (_("Provider %s does not implement entry point"),
-				       dsn_info->provider);
+			emit_client_error (client, NULL,
+					   _("Provider %s does not implement entry function"),
+					   dsn_info->provider);
 			gda_config_free_data_source_info (dsn_info);
 			g_free (prv);
 			return NULL;
@@ -275,7 +305,8 @@ gda_client_open_connection (GdaClient *client,
 
 		prv->provider = prv->plugin_create_provider ();
 		if (!prv->provider) {
-			gda_log_error (_("Creation of GdaServerProvider failed"));
+			emit_client_error (client, NULL,
+					   _("Could not create GdaServerProvider object from plugin"));
 			gda_config_free_data_source_info (dsn_info);
 			g_free (prv);
 			return NULL;
@@ -450,3 +481,107 @@ gda_client_close_all_connections (GdaClient *client)
 	g_list_free (client->priv->connections);
 	client->priv->connections = NULL;
 }
+
+/**
+ * gda_client_begin_transaction
+ * @client: a #GdaClient object.
+ * @xaction: a #GdaTransaction object.
+ *
+ * Start a transaction on all connections being managed by the given
+ * #GdaClient. It is important to note that this operates on all
+ * connections opened within a #GdaClient, which could not be what
+ * you're looking for.
+ *
+ * To execute a transaction on a unique connection, use
+ * #gda_connection_begin_transaction, #gda_connection_commit_transaction
+ * and #gda_connection_rollback_transaction.
+ *
+ * Returns: TRUE if all transactions could be started successfully,
+ * or FALSE if one of them fails.
+ */
+gboolean
+gda_client_begin_transaction (GdaClient *client, GdaTransaction *xaction)
+{
+	GList *l;
+
+	g_return_val_if_fail (GDA_IS_CLIENT (client), FALSE);
+	g_return_val_if_fail (GDA_IS_TRANSACTION (xaction), FALSE);
+
+	for (l = client->priv->connections; l != NULL; l = l->next) {
+		if (!gda_connection_begin_transaction (GDA_CONNECTION (l->data), xaction)) {
+			gda_client_rollback_transaction (client, xaction);
+			return FALSE;
+		}
+	}
+
+	return TRUE;
+}
+
+/**
+ * gda_client_commit_transaction
+ * @client: a #GdaClient object.
+ * @xaction: a #GdaTransaction object.
+ *
+ * Commit a running transaction on all connections being managed by the given
+ * #GdaClient. It is important to note that this operates on all
+ * connections opened within a #GdaClient, which could not be what
+ * you're looking for.
+ *
+ * To execute a transaction on a unique connection, use
+ * #gda_connection_begin_transaction, #gda_connection_commit_transaction
+ * and #gda_connection_rollback_transaction.
+ *
+ * Returns: TRUE if all transactions could be committed successfully,
+ * or FALSE if one of them fails.
+ */
+gboolean
+gda_client_commit_transaction (GdaClient *client, GdaTransaction *xaction)
+{
+	GList *l;
+
+	g_return_val_if_fail (GDA_IS_CLIENT (client), FALSE);
+	g_return_val_if_fail (GDA_IS_TRANSACTION (xaction), FALSE);
+
+	for (l = client->priv->connections; l != NULL; l = l->next) {
+		if (!gda_connection_commit_transaction (GDA_CONNECTION (l->data), xaction)) {
+			gda_client_rollback_transaction (client, xaction);
+			return FALSE;
+		}
+	}
+
+	return TRUE;
+}
+
+/**
+ * gda_client_rollback_transaction
+ * @client: a #GdaClient object.
+ * @xaction: a #GdaTransaction object.
+ *
+ * Cancels a running transaction on all connections being managed by the given
+ * #GdaClient. It is important to note that this operates on all
+ * connections opened within a #GdaClient, which could not be what
+ * you're looking for.
+ *
+ * To execute a transaction on a unique connection, use
+ * #gda_connection_begin_transaction, #gda_connection_commit_transaction
+ * and #gda_connection_rollback_transaction.
+ *
+ * Returns: TRUE if all transactions could be cancelled successfully,
+ * or FALSE if one of them fails.
+ */
+gboolean
+gda_client_rollback_transaction (GdaClient *client, GdaTransaction *xaction)
+{
+	GList *l;
+	gint failures = 0;
+
+	g_return_val_if_fail (GDA_IS_CLIENT (client), FALSE);
+	g_return_val_if_fail (GDA_IS_TRANSACTION (xaction), FALSE);
+
+	for (l = client->priv->connections; l != NULL; l = l->next) {
+		if (!gda_connection_rollback_transaction (GDA_CONNECTION (l->data), xaction))
+			failures++;
+	}
+
+	return failures == 0 ? TRUE : FALSE;
+}
